#include "simpletools.h"    // Include simpletools library
#include "fdserial.h"       // Include Full Duplex Serial library

#define Reboot() __builtin_propeller_clkset(0x80)

//volatile char lock;

#define CALC_STACK_SIZE 64
#define INJ1_STACK_SIZE	64

#define CONDITION_1	// Calc cnt vars locally defined, NOT volatile <- This does not work, should be faster and smaller
//#define CONDITION_2	// Calc cnt vars locally defined as volatile <- Works but a little slower than 3
//#define CONDITION_3	// Calc cnt vars globally defined as volatile <- Works.

#ifdef CONDITION_3
volatile uint_fast32_t calc_start_cnt = 0;
volatile uint_fast32_t calc_end_cnt = 0;
#endif

// Function prototype and stack guards for calc cog
void calc(void *par);
static uint_fast32_t calc_stack_guard1 = 0xDEADBEEF;	// Unique value placed in mem location just before stack
static uint_fast32_t calc_stack[CALC_STACK_SIZE];		//
static uint_fast32_t calc_stack_guard2 = 0xDEADBEEF;	// Unique value placed in mem location just after stack

// Function prototype and stack guards for inj1 cog
void inj1(void *par);
static uint_fast32_t inj1_stack_guard1 = 0xDEADBEEF;
static uint_fast32_t inj1_stack[INJ1_STACK_SIZE];
static uint_fast32_t inj1_stack_guard2 = 0xDEADBEEF;

// function prototype for test cog
void test(void *par);
static uint_fast32_t test_stack[64];

// ADCChanPri - sets priority for each ADC channel.
// Use: ADCChanPri[x] = y;
// Ex: ADCChanPri[6] = 500;
// Channel 6 will be read every 500 loops of the calc cog code.
volatile uint_fast32_t ADCChanPri[8] = {1,2,4,8,16,32,64,128};

volatile uint_fast32_t adcval[8]; // Contains most recent adc values





/* EEPROM / Hub Memory Structures */

/* 16 bit variables are good compromise between granularity, space and speed. */


volatile uint_fast32_t FuelBaseTable[21][21]; // 882 bytes
volatile uint_fast32_t FuelBaseTable_org = 32768;
// Occupies 32768 to 33649


volatile uint_fast32_t IgnBaseTable[21][21];  //882 bytes
volatile uint_fast32_t IgnBaseTable_org = 33650;
// Occupies 33650 to 34531


// CLT_COMP

volatile uint_fast32_t CLTCompTable[13][13]; //338 bytes
volatile uint_fast32_t CLTCompTable_org = 34532;
// Occupies 34532 to 34869

// MAT_COMP

volatile uint_fast32_t MATCompTable[13][13];  //338 bytes
volatile uint_fast32_t MATCompTable_org = 34870;
// Occupies 34870 to 35207


// Channel Enable
volatile char Chan_Enab[24];    // 24 bytes
volatile uint_fast32_t Chan_Enab_org = 35208;
// Occupies 35208 to 35231

// Primary injector static flow rates
volatile uint_fast32_t Inj_Flow_Pri; // 2 bytes
volatile uint_fast32_t Inj_Flow_Pri_org = 35232;
// Occupies 35232 to 35233

// Secondary injector static flow rates
volatile uint_fast32_t Inj_Flow_Sec; // 2 bytes
volatile uint_fast32_t Inj_Flow_Sec_org = 35234;
// Occupies 35234 to 35235

// AFR Target Table
volatile uint_fast32_t AFRTable[21][21]; // 882 bytes
volatile uint_fast32_t AFRTable_org = 35236;
// Occupies 35236 to 36118


// Total in-hub table usage = 36118 - 32768 = 3350 bytes.

volatile uint_fast32_t RTV[24];      // 24 channels of RT Var Values






// Pin definitions
volatile uint_fast32_t CEL = 16;
//volatile uint_fast32_tp17 = 17;
volatile uint_fast32_tp1 = 1;


volatile uint_fast32_t x1,x2,y1,y2,r,r2,r3;
volatile uint_fast32_t cnts_per_calc_loop = 0,cnts_per_test_loop = 0;



volatile uint_fast32_t calcloops = 0;
volatile uint_fast32_t testloops = 0;


volatile uint_fast32_t rpm = 2500;
volatile uint_fast32_t map = 50, baro = 0, mat = 0, batt = 0, clt = 0;
volatile uint_fast32_t fuelresult,et,st,ft;
volatile uint_fast32_t rpm_y;

volatile uint_fast32_t yl,ul,ur,ll,lr,hx,hy,lx,ly;







/*
				SYSTEM SUPERVISOR
				-----------------
	A) This code is the boot code. It's loaded into Cog 0 and ...

		1) Defines and initializes global variables.

		2) Starts the serial port driver, fdserial (Full Duplex Serial) running in it's own cog.

		3) At boot, copies various data structures from nonvolatile storage (the upper 32K of EEPROM) into hub memory locations for use by the rest of the code.

		4) Provides read/write access between the PC based tuning application and the hub memory to facilitate editing of live data structures. This provides realtime tuning capability.

		5) Commands are provided to store live data to nonvolatile storage, the EEPROM.

		6) Provides for streaming of tuning app selectable Run Time Variables such as RPM,MAP, etc. up to 24 channels of data for logging and/or real time observation of these values.

	B) Starts and/or stops the other cogs and tasks as needed. Cog use is:

		0)	Supervisory functions as described above.

		1)	Full Duplex Serial Driver.

		2)	Free running loop that performs table lookup duties and ...
			ADC value acquisition of all analog values; averaging and smoothing to be done as
			each value requires. Each ADC channel can have it's priority changed.
			Averaging and smoothing parameters to be exposed to PC based
			Tuning Application.
			Bilinear interpolation routines.
			Base timing and ign values and all ADC values are exposed globally for further
			processing before implementation into commanded actions.

		3)  Angle clock. Hopefully this code can fit into cog memory though LMM code should suffice.

		5)	TBD

		6)	TBD

		7)  TBD



*/






uint_fast32_t main()
    {
    uint_fast32_t i,i2;
    char b,b1,b2,c,rcv_c;
    //char cksum;
    uint_fast32_t si1,si_accum;
	uint_fast32_t temp1;
    //uint_fast32_tta_str_org = 34712; // EEPROM addr of 24x12 chars - RTV names for PA App

    fdserial *serial = fdserial_open(31,30,0,350000);  // Fire up serial port interface


//------------------------------ calc cog startup
    memset(calc_stack, 0xA5, sizeof(calc_stack));	// Fill stack w/ unique value to detect amt used
    // See testcase();
    cogstart(&calc, NULL, calc_stack, sizeof(calc_stack)); // Start calc cog


//------------------------------ test cog startup
//    cogstart(&test, NULL, test_stack, sizeof(test_stack)); // Start test cog

//------------------------------ inj1 cog startup
//    memset(inj1_stack, 0xA5, sizeof(inj1_stack));	// Fill stack w/ unique value to detect amt used
    // See testcase();
//    cogstart(&inj1, NULL, inj1_stack, sizeof(inj1_stack));	// Start inj1 cog - does nothing
    // currently.



    /* %%%%%%%%%%%%%%% FuelBaseTable Routines %%%%%%%%%%%%%%%*/

    void FuelBaseTable_EE2Mem()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 =0; i2 < 21; i2++ )
                {
                b1=ee_getByte(FuelBaseTable_org+(i*42)+(i2*2));
                b2=ee_getByte(FuelBaseTable_org+(i*42)+(i2*2)+1);
                FuelBaseTable[i2][i] = (b1<<8)+b2;
                }
            }
        }


    void FuelBaseTable_Mem2EE()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 = 0; i2 < 21; i2++ )
                {
                b1=FuelBaseTable[i2][i]>>8;
                b2=FuelBaseTable[i2][i];
                ee_putByte(b1,FuelBaseTable_org+(i*42)+(i2*2));
                ee_putByte(b2,FuelBaseTable_org+(i*42)+(i2*2)+1);
                }
            }
        }

    void FuelBaseTable_Mem2App()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 = 0; i2 < 21; i2++ )
                {
                si1 = FuelBaseTable[i][i2];
                b = si1>>8;
                fdserial_txChar(serial,b);
                b = si1;
                fdserial_txChar(serial,b);
                pause(1);
                }
            }
        }

    void FuelBaseTable_App2Mem()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 = 0; i2 < 21; i2++ )
                {
                si_accum = 0;
                rcv_c = fdserial_rxChar(serial);
                si_accum = (rcv_c << 8);
                rcv_c = fdserial_rxChar(serial);
                si_accum = si_accum + (rcv_c);
                FuelBaseTable[i][i2] = si_accum;
                }
            }
        }


    /* %%%%%%%%%%%%%%% IgnBaseTable Routines %%%%%%%%%%%%%%%*/

    void IgnBaseTable_EE2Mem()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 =0; i2 < 21; i2++ )
                {
                b1=ee_getByte(IgnBaseTable_org+(i*42)+(i2*2));
                b2=ee_getByte(IgnBaseTable_org+(i*42)+(i2*2)+1);
                IgnBaseTable[i][i2] = (b1<<8)+b2;
                }
            }
        }


    void IgnBaseTable_Mem2EE()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 = 0; i2 < 21; i2++ )
                {
                b1=IgnBaseTable[i][i2]>>8;
                b2=IgnBaseTable[i][i2];
                ee_putByte(b1,IgnBaseTable_org+(i*42)+(i2*2));
                ee_putByte(b2,IgnBaseTable_org+(i*42)+(i2*2)+1);
                }
            }
        }

    void IgnBaseTable_Mem2App()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 = 0; i2 < 21; i2++ )
                {
                si1 = IgnBaseTable[i][i2];
                b = si1>>8;
                fdserial_txChar(serial,b);
                b = si1;
                fdserial_txChar(serial,b);
                pause(1);
                }
            }
        }






    void IgnBaseTable_App2Mem()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 = 0; i2 < 21; i2++ )
                {
                si_accum = 0;
                rcv_c = fdserial_rxChar(serial);
                si_accum = (rcv_c << 8);
                rcv_c = fdserial_rxChar(serial);
                si_accum = si_accum + (rcv_c);
                IgnBaseTable[i][i2] = si_accum;
                }
            }
        }


    /* %%%%%%%%%%%%%%% AFRTable Routines %%%%%%%%%%%%%%%*/

    void AFRTable_EE2Mem()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 =0; i2 < 21; i2++ )
                {
                b1=ee_getByte(AFRTable_org+(i*42)+(i2*2));
                b2=ee_getByte(AFRTable_org+(i*42)+(i2*2)+1);
                AFRTable[i][i2] = (b1<<8)+b2;
                }
            }
        }


    void AFRTable_Mem2EE()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 = 0; i2 < 21; i2++ )
                {
                b1=AFRTable[i][i2]>>8;
                b2=AFRTable[i][i2];
                ee_putByte(b1,AFRTable_org+(i*42)+(i2*2));
                ee_putByte(b2,AFRTable_org+(i*42)+(i2*2)+1);
                }
            }
        }

    void AFRTable_Mem2App()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 = 0; i2 < 21; i2++ )
                {
                si1 = AFRTable[i][i2];
                b = si1>>8;
                fdserial_txChar(serial,b);
                b = si1;
                fdserial_txChar(serial,b);
                pause(1);
                }
            }
        }

    void AFRTable_App2Mem()
        {
        for (i = 0; i < 21; i++ )
            {
            for (i2 = 0; i2 < 21; i2++ )
                {
                si_accum = 0;
                rcv_c = fdserial_rxChar(serial);
                si_accum = (rcv_c << 8);
                rcv_c = fdserial_rxChar(serial);
                si_accum = si_accum + (rcv_c);
                AFRTable[i][i2] = si_accum;
                }
            }
        }


    /* %%%%%%%%%%%%%%% CLTCompTable Routines %%%%%%%%%%%%%%%*/

    void CLTCompTable_EE2Mem()
        {
        for (i = 0; i < 13; i++ )
            {
            for (i2 =0; i2 < 13; i2++ )
                {
                b1=ee_getByte(CLTCompTable_org+(i*26)+(i2*2));
                b2=ee_getByte(CLTCompTable_org+(i*26)+(i2*2)+1);
                CLTCompTable[i][i2] = (b1<<8)+b2;
                }
            }
        }

    void CLTCompTable_Mem2EE()
        {
        for (i = 0; i < 13; i++ )
            {
            for (i2 = 0; i2 < 13; i2++ )
                {
                b1=CLTCompTable[i][i2]>>8;
                b2=CLTCompTable[i][i2];
                ee_putByte(b1,CLTCompTable_org+(i*26)+(i2*2));
                ee_putByte(b2,CLTCompTable_org+(i*26)+(i2*2)+1);
                }
            }
        }


    void CLTCompTable_Mem2App()
        {
        }

    void CLTCompTable_App2Mem()
        {
        }

    /* %%%%%%%%%%%%%%% MATCompTable Routines %%%%%%%%%%%%%%%*/

    void MATCompTable_EE2Mem()
        {
        for (i = 0; i < 13; i++ )
            {
            for (i2 =0; i2 < 13; i2++ )
                {
                b1=ee_getByte(MATCompTable_org+(i*26)+(i2*2));
                b2=ee_getByte(MATCompTable_org+(i*26)+(i2*2)+1);
                MATCompTable[i][i2] = (b1<<8)+b2;
                }
            }
        }

    void MATCompTable_Mem2EE()
        {
        for (i = 0; i < 13; i++ )
            {
            for (i2 = 0; i2 < 13; i2++ )
                {
                b1=MATCompTable[i][i2]>>8;
                b2=MATCompTable[i][i2];
                ee_putByte(b1,MATCompTable_org+(i*26)+(i2*2));
                ee_putByte(b2,MATCompTable_org+(i*26)+(i2*2)+1);
                }
            }
        }


    void MATCompTable_Mem2App()
        {
        }

    void MATCompTable_App2Mem()
        {
        }



    /* %%%%%%%%%%%%%%% Chan_Enable Routines %%%%%%%%%%%%%%%*/



    void Chan_Enab_EE2Mem()
        {
        for (i =0; i < 24; i++ )
            {
            Chan_Enab[i] = ee_getByte(Chan_Enab_org+i);
            }
        }



    void Chan_Enab_Mem2EE()
        {
        for (i = 0; i < 24; i++ )
            {
            ee_putByte(Chan_Enab[i],Chan_Enab_org+i);
            }

        }

    void Chan_Enab_Mem2App()
        {
        }

    void Chan_Enab_App2Mem()
        {
        }



    /* %%%%%%%%%%%%%%% Inj_Flow Routines %%%%%%%%%%%%%%%*/

    void Inj_Flow_EE2Mem()
        {
        b1 = ee_getByte(Inj_Flow_Pri_org);
        b2 = ee_getByte(Inj_Flow_Pri_org+1);
        Inj_Flow_Pri = (b1<<8)+b2;
        b1 = ee_getByte(Inj_Flow_Sec_org);
        b2 = ee_getByte(Inj_Flow_Sec_org+1);
        Inj_Flow_Sec = (b1<<8)+b2;
        }

    void Inj_Flow_Mem2EE()
        {
        b1 = Inj_Flow_Pri>>8;
        b2 = Inj_Flow_Pri;
        ee_putByte(b1,Inj_Flow_Pri_org);
        ee_putByte(b2,Inj_Flow_Pri_org+1);
        b1 = Inj_Flow_Sec>>8;
        b2 = Inj_Flow_Sec;
        ee_putByte(b1,Inj_Flow_Sec_org);
        ee_putByte(b2,Inj_Flow_Sec_org+1);
        }

    void Inj_Flow_Mem2App()
        {
        }

    void Inj_Flow_App2Mem()
        {
        }


    /*
    void CntReport()
    			{
    				li = 0;
    				PHSA = 0;

    				// Init CTRA
    				CTRA = (0b00001<<26);
    				FRQA = 10;
    				pause(4);
    				//li = CNT;
    				li = PHSA;
    				b = li>>24;
    				fdserial_txChar(serial,b);
    				b = li>>16;
    				fdserial_txChar(serial,b);
    				b = li>>8;
    				fdserial_txChar(serial,b);
    				b = li;
    				fdserial_txChar(serial,b);
    				pause(1);
    				CTRA = (0b00000<<26);
    			}
    */

// Debugging stuff

    void sendint(uint_fast32_t x)
        {
        b = x>>24;
        fdserial_txChar(serial,b);
        b = x>>16;
        fdserial_txChar(serial,b);
        b = x>>8;
        fdserial_txChar(serial,b);
        b = x;
        fdserial_txChar(serial,b);
        }

    void sendshortint(uint_fast32_t x)
        {
        b = x>>8;
        fdserial_txChar(serial,b);
        b = x;
        fdserial_txChar(serial,b);
        }

    void ECUStats()
        {
        //while (lock == 1); // Wait for lock to open
            {
            //lock = 1;      // Set lock
			//waitcnt(CLKFREQ/1000+CNT);
            sendint(rpm);
            sendint(map);
            sendint(fuelresult);
            sendint(cnts_per_calc_loop);
			temp1 = 100000000/cnts_per_calc_loop;
			sendint(temp1);
			sendint(cnts_per_test_loop);
			temp1 = 100000000/cnts_per_test_loop;
            sendint(temp1);

            uint_fast32_t j,k;
            for (j = 0;  j < CALC_STACK_SIZE; j++)
                if (calc_stack[j] != 0xA5A5A5A5)
                    break;
            k = CALC_STACK_SIZE - j;
            sendint(k);
            k=0;
            if (calc_stack_guard1 != 0xDEADBEEF)
                k=1;
            if (calc_stack_guard2 != 0xDEADBEEF)
                k=k+2;
            sendint(k);
            sendint(CALC_STACK_SIZE);

            for (j = 0;  j < INJ1_STACK_SIZE; j++)
                if (inj1_stack[j] != 0xA5A5A5A5)
                    break;
            k = INJ1_STACK_SIZE - j;
            sendint(k);
            k=0;
            if (inj1_stack_guard1 != 0xDEADBEEF)
                k=1;
            if (inj1_stack_guard2 != 0xDEADBEEF)
                k=k+2;
            sendint(k);
            sendint(INJ1_STACK_SIZE);

			sendint(calcloops);
			sendint(testloops);

            //lock = 0;   // Clear lock
            }
        }

    /* === Copy EEPROM locations into hub memory === */

    FuelBaseTable_EE2Mem();
    IgnBaseTable_EE2Mem();
//clttemp_2hub();
//cltcorr_2hub();
//voltvolt_2hub();
//voltcorr_2hub();
//fueltable_2hub();
//igntable_2hub();
//chanenab_2hub();





    /* =================================================================== */

    /* =================================================================== */
    /* For debug purposes - populate chan array */
//    for (i =0; i < 24; i++ )  {
//      RTV[i] = i*10+10;
    //Chan_Enab[i] = 1;
//    }
    /* =================================================================== */



    while(1)
        {
        fdserial_rxFlush(serial);
        c = fdserial_rxChar(serial);

        switch (c)
            {

            case 1 :
                {

                ECUStats();
                }
            break;

            case 130 :                   // Ack PC App.
                {
                fdserial_txChar(serial,160);
                fdserial_rxFlush(serial);  // Clear rcv buffer from extraneous crap
                }
            break;

            /* +++++++++++++ FuelBaseTable Management */

            case 10 :
                {
                FuelBaseTable_EE2Mem();
                }
            break;

            case 11 :
                {
                FuelBaseTable_Mem2EE();
                }
            break;

            case 12 :
                {
                FuelBaseTable_Mem2App();
                }
            break;

            case 13 :
                {
                FuelBaseTable_App2Mem();
                }
            break;

            /* +++++++++++++ IgnBaseTable Management */

            case 20 :
                {
                IgnBaseTable_EE2Mem();
                }
            break;

            case 21 :
                {
                IgnBaseTable_Mem2EE();
                }
            break;

            case 22 :
                {
                IgnBaseTable_Mem2App();
                }
            break;

            case 23 :
                {
                IgnBaseTable_App2Mem();
                }
            break;

            /* +++++++++++++ AFRTable Management */

            case 30 :
                {
                AFRTable_EE2Mem();
                }
            break;

            case 31 :
                {
                AFRTable_Mem2EE();
                }
            break;

            case 32 :
                {
                AFRTable_Mem2App();
                }
            break;

            case 33 :
                {
                AFRTable_App2Mem();
                }
            break;



            /* +++++++++++++ CLTCompTable Management */

            case 40 :
                {
                CLTCompTable_EE2Mem();
                }
            break;

            case 41 :
                {
                CLTCompTable_Mem2EE();
                }
            break;

            case 42 :
                {
                CLTCompTable_Mem2App();
                }
            break;

            case 43 :
                {
                CLTCompTable_App2Mem();
                }
            break;


            /* +++++++++++++ MATCompTable Management */

            case 50 :
                {
                CLTCompTable_EE2Mem();
                }
            break;

            case 51 :
                {
                CLTCompTable_Mem2EE();
                }
            break;

            case 52 :
                {
                CLTCompTable_Mem2App();
                }
            break;

            case 53 :
                {
                CLTCompTable_App2Mem();
                }
            break;


            case 255 :
                {
                Reboot();
                }
            break;




            }
        }

    }


void adc(void *par)
    {
    uint_fast32_t adcloops;
    while(1)
        {

        }
    }


__attribute__((fcache))			// Runs completely within cog. Yes!!!
void inj1(void *par)            // Totally test code to see how counters are best
    {
    // implemented in PropGCC.
    DIRA |= (1 << 1);
    CTRA = (0b00100<<26)+1;
    FRQA = 1;
    while(1)
        {
        PHSA = -10000; // for 100 uSec pulse
        //pause(1);
        waitcnt(CLKFREQ/1000+CNT);
        }
    }






__attribute__((optimize("-Os")))
void calc(void *par)
    {
    uint_fast32_t ix,jx;
    int_fast32_t interp1, interp2, interp3;
    uint_fast32_t adcloopcnt[8],loop;
#ifdef CONDITION_1
    uint_fast32_t calc_start_cnt = 0;
    uint_fast32_t calc_end_cnt = 0;
#endif
#ifdef CONDITION_2
    volatile uint_fast32_t calc_start_cnt = 0;
    volatile uint_fast32_t calc_end_cnt = 0;
#endif


    while(1)
        {
        //while (lock == 1); // Wait for lock to open
            {
            //lock = 1;      // Set lock
			//waitcnt(CLKFREQ/500+CNT);
			calcloops++;
            calc_start_cnt = CNT;
			//rpm = 2500; map = 50;


            for (loop = 0; loop<9 ; loop++ )
                {
                if ((adcloopcnt[loop] == ADCChanPri[loop]))
                    {
                    //adcval[loop] = readADCAverage(loop,2,3,4,5);
                    adcloopcnt[loop] = 0;
                    }
                else
                    {
                    adcloopcnt[loop]++;
                    }
                }





            if(rpm > FuelBaseTable[0][20])
                {
                rpm = FuelBaseTable[0][20];
                }
            else if(rpm < FuelBaseTable[0][1])
                {
                rpm = FuelBaseTable[0][1];
                }
            if(map > FuelBaseTable[20][0])
                {
                map = FuelBaseTable[20][0];
                }
            else if(map < FuelBaseTable[1][0])
                {
                map = FuelBaseTable[1][0];
                }
            for (ix = 20-2; ix>-1; ix--)
                {
                if (map>FuelBaseTable[ix][0])
                    {
                    break;
                    }
                }

            if(ix < 0)ix = 0;

            for (jx = 20-2; jx > -1; jx--)
                {
                if (rpm>FuelBaseTable[0][jx])
                    {
                    break;
                    }
                }

            if(jx < 0)jx = 0;

            interp1 = FuelBaseTable[ix + 1][0] - FuelBaseTable[ix][0];
            if(interp1 != 0)
                {
                interp3 = (map - FuelBaseTable[ix][0]);
                interp3 = (100 * interp3);
                interp1 = interp3 / interp1;
                }

            interp2 =	FuelBaseTable[0][jx + 1] - FuelBaseTable[0][jx];
            if(interp2 != 0)
                {
                interp3 = (rpm - FuelBaseTable[0][jx]);
                interp3 = (100 * interp3);
                interp2 = interp3 / interp2;
                }

            fuelresult = ((100 - interp1) * (100 - interp2) * FuelBaseTable[ix][jx]
                            + interp1 * (100 - interp2) * FuelBaseTable[ix+1][jx]
                            + interp2 * (100 - interp1) * FuelBaseTable[ix][jx+1]
                            + interp1 * interp2 * FuelBaseTable[ix+1][jx+1]) / 10000;


            calc_end_cnt = CNT;
            cnts_per_calc_loop = calc_end_cnt - calc_start_cnt;

            //lock = 0;
            }
        }
    }


void test(void *par)
    {
    volatile uint_fast32_t test_start_cnt = 0;
    volatile uint_fast32_t test_end_cnt = 0;
    while(1)
        {
        //while (lock == 1); // Wait for lock to open
            {
            //lock = 1;
			//waitcnt(CLKFREQ/1000+CNT);
			testloops++;
            test_start_cnt = CNT;
            //pause(2);
            test_end_cnt = CNT;
            cnts_per_test_loop = test_end_cnt - test_start_cnt;
            //pause(1);
            //lock = 0;
            }

        }
    }
